@@ -71,12 +71,15 @@ static void _cleanup(void);
 static enum oonf_telnet_result _cb_layer2info(struct oonf_telnet_data *con);
 static enum oonf_telnet_result _cb_layer2info_help(struct oonf_telnet_data *con);
 
-static void _initialize_interface_values(struct oonf_layer2_net *net);
-static void _initialize_net_data_values(struct oonf_viewer_template *template,
-    struct isonumber_str *dst, struct oonf_layer2_data *data);
-static void _initialize_net_data_values(struct oonf_viewer_template *template,
-    struct isonumber_str *dst, struct oonf_layer2_data *data);
-static void _initialize_neighbor_values(struct oonf_layer2_neigh *neigh);
+static void _initialize_if_data_values(struct oonf_viewer_template *template,
+    struct oonf_layer2_data *data);
+static void _initialize_if_origin_values(struct oonf_layer2_data *data);
+static void _initialize_if_values(struct oonf_layer2_net *net);
+
+static void _initialize_neigh_data_values(struct oonf_viewer_template *template,
+    struct oonf_layer2_data *data);
+static void _initialize_neigh_origin_values(struct oonf_layer2_data *data);
+static void _initialize_neigh_values(struct oonf_layer2_neigh *neigh);
 
 static int _cb_create_text_interface(struct oonf_viewer_template *);
 static int _cb_create_text_neighbor(struct oonf_viewer_template *);
@@ -98,6 +101,9 @@ static int _cb_create_text_dst(struct oonf_viewer_template *);
 /*! template key for interface type */
 #define KEY_IF_TYPE                     "if_type"
 
+/*! template key for DLEP interface */
+#define KEY_IF_DLEP                     "if_dlep"
+
 /*! template key for interface identifier */
 #define KEY_IF_IDENT                    "if_ident"
 
@@ -119,6 +125,9 @@ static int _cb_create_text_dst(struct oonf_viewer_template *);
 /*! template key for destination address */
 #define KEY_DST_ADDR                    "dst_addr"
 
+/*! template key for destination origin */
+#define KEY_DST_ORIGIN                  "dst_origin"
+
 
 /*! string prefix for all interface keys */
 #define KEY_IF_PREFIX                   "if_"
@@ -126,6 +135,9 @@ static int _cb_create_text_dst(struct oonf_viewer_template *);
 /*! string prefix for all neighbor keys */
 #define KEY_NEIGH_PREFIX                "neigh_"
 
+/*! string suffix for all data originators */
+#define KEY_ORIGIN_SUFFIX               "_origin"
+
 /*
  * buffer space for values that will be assembled
  * into the output of the plugin
@@ -133,17 +145,20 @@ static int _cb_create_text_dst(struct oonf_viewer_template *);
 static char                             _value_if[IF_NAMESIZE];
 static char                             _value_if_index[12];
 static char                             _value_if_type[16];
+static char                             _value_if_dlep[TEMPLATE_JSON_BOOL_LENGTH];
 static char                             _value_if_ident[33];
 static struct netaddr_str               _value_if_ident_addr;
 static struct netaddr_str               _value_if_local_addr;
 static struct isonumber_str             _value_if_lastseen;
 static struct isonumber_str             _value_if_data[OONF_LAYER2_NET_COUNT];
-
+static char                             _value_if_origin[OONF_LAYER2_NET_COUNT][IF_NAMESIZE];
 static struct netaddr_str               _value_neigh_addr;
 static struct isonumber_str             _value_neigh_lastseen;
 static struct isonumber_str             _value_neigh_data[OONF_LAYER2_NEIGH_COUNT];
+static char                             _value_neigh_origin[OONF_LAYER2_NEIGH_COUNT][IF_NAMESIZE];
 
 static struct netaddr_str               _value_dst_addr;
+static char                             _value_dst_origin[IF_NAMESIZE];
 
 /* definition of the template data entries for JSON and table output */
 static struct abuf_template_data_entry _tde_if_key[] = {
@@ -154,12 +169,14 @@ static struct abuf_template_data_entry _tde_if_key[] = {
 
 static struct abuf_template_data_entry _tde_if[] = {
     { KEY_IF_TYPE, _value_if_type, true },
+    { KEY_IF_DLEP, _value_if_dlep, true },
     { KEY_IF_IDENT, _value_if_ident, true },
     { KEY_IF_IDENT_ADDR, _value_if_ident_addr.buf, true },
     { KEY_IF_LASTSEEN, _value_if_lastseen.buf, false },
 };
 
 static struct abuf_template_data_entry _tde_if_data[OONF_LAYER2_NET_COUNT];
+static struct abuf_template_data_entry _tde_if_origin[OONF_LAYER2_NET_COUNT];
 
 static struct abuf_template_data_entry _tde_neigh_key[] = {
     { KEY_NEIGH_ADDR, _value_neigh_addr.buf, true },
@@ -170,10 +187,14 @@ static struct abuf_template_data_entry _tde_neigh[] = {
 };
 
 static struct abuf_template_data_entry _tde_neigh_data[OONF_LAYER2_NEIGH_COUNT];
+static struct abuf_template_data_entry _tde_neigh_origin[OONF_LAYER2_NEIGH_COUNT];
 
 static struct abuf_template_data_entry _tde_dst_key[] = {
     { KEY_DST_ADDR, _value_dst_addr.buf, true },
 };
+static struct abuf_template_data_entry _tde_dst[] = {
+    { KEY_DST_ORIGIN, _value_dst_origin, true },
+};
 
 static struct abuf_template_storage _template_storage;
 static struct autobuf _key_storage;
@@ -183,21 +204,25 @@ static struct abuf_template_data _td_if[] = {
     { _tde_if_key, ARRAYSIZE(_tde_if_key) },
     { _tde_if, ARRAYSIZE(_tde_if) },
     { _tde_if_data, ARRAYSIZE(_tde_if_data) },
+    { _tde_if_origin, ARRAYSIZE(_tde_if_origin) },
 };
 static struct abuf_template_data _td_neigh[] = {
     { _tde_if_key, ARRAYSIZE(_tde_if_key) },
     { _tde_neigh_key, ARRAYSIZE(_tde_neigh_key) },
     { _tde_neigh, ARRAYSIZE(_tde_neigh) },
     { _tde_neigh_data, ARRAYSIZE(_tde_neigh_data) },
+    { _tde_neigh_origin, ARRAYSIZE(_tde_neigh_origin) },
 };
 static struct abuf_template_data _td_default[] = {
     { _tde_if_key, ARRAYSIZE(_tde_if_key) },
     { _tde_neigh_data, ARRAYSIZE(_tde_neigh_data) },
+    { _tde_neigh_origin, ARRAYSIZE(_tde_neigh_origin) },
 };
 static struct abuf_template_data _td_dst[] = {
     { _tde_if_key, ARRAYSIZE(_tde_if_key) },
     { _tde_neigh_key, ARRAYSIZE(_tde_neigh_key) },
     { _tde_dst_key, ARRAYSIZE(_tde_dst_key) },
+    { _tde_dst, ARRAYSIZE(_tde_dst) },
 };
 
 /* OONF viewer templates (based on Template Data arrays) */
@@ -272,6 +297,16 @@ _init(void) {
     abuf_puts(&_key_storage, KEY_IF_PREFIX);
     abuf_puts(&_key_storage, oonf_layer2_get_net_metadata(i)->key);
     abuf_memcpy(&_key_storage, "\0", 1);
+
+    _tde_if_origin[i].key =
+        abuf_getptr(&_key_storage) + abuf_getlen(&_key_storage);
+    _tde_if_origin[i].value = _value_if_origin[i];
+    _tde_if_origin[i].string = true;
+
+    abuf_puts(&_key_storage, KEY_IF_PREFIX);
+    abuf_puts(&_key_storage, oonf_layer2_get_net_metadata(i)->key);
+    abuf_puts(&_key_storage, KEY_ORIGIN_SUFFIX);
+    abuf_memcpy(&_key_storage, "\0", 1);
   }
 
   for (i=0; i<OONF_LAYER2_NEIGH_COUNT; i++) {
@@ -283,6 +318,16 @@ _init(void) {
     abuf_puts(&_key_storage, KEY_NEIGH_PREFIX);
     abuf_puts(&_key_storage, oonf_layer2_get_neigh_metadata(i)->key);
     abuf_memcpy(&_key_storage, "\0", 1);
+
+    _tde_neigh_origin[i].key =
+        abuf_getptr(&_key_storage) + abuf_getlen(&_key_storage);
+    _tde_neigh_origin[i].value = _value_neigh_origin[i];
+    _tde_neigh_origin[i].string = true;
+
+    abuf_puts(&_key_storage, KEY_NEIGH_PREFIX);
+    abuf_puts(&_key_storage, oonf_layer2_get_neigh_metadata(i)->key);
+    abuf_puts(&_key_storage, KEY_ORIGIN_SUFFIX);
+    abuf_memcpy(&_key_storage, "\0", 1);
   }
 
   oonf_telnet_add(&_telnet_commands[0]);
@@ -327,15 +372,17 @@ _cb_layer2info_help(struct oonf_telnet_data *con) {
  * @param net pointer to layer2 interface
  */
 static void
-_initialize_interface_values(struct oonf_layer2_net *net) {
-  struct os_interface_data *data;
+_initialize_if_values(struct oonf_layer2_net *net) {
+  struct os_interface *os_if;
 
-  data = &net->if_listener.interface->data;
+  os_if = net->if_listener.data;
 
   strscpy(_value_if, net->name, sizeof(_value_if));
-  snprintf(_value_if_index, sizeof(_value_if_index), "%u", data->index);
+  snprintf(_value_if_index, sizeof(_value_if_index), "%u", os_if->index);
   strscpy(_value_if_ident, net->if_ident, sizeof(_value_if_ident));
-  netaddr_to_string(&_value_if_local_addr, &data->mac);
+  netaddr_to_string(&_value_if_local_addr, &os_if->mac);
+  strscpy(_value_if_type, oonf_layer2_get_network_type(net->if_type), IF_NAMESIZE);
+  strscpy(_value_if_dlep, json_getbool(net->if_dlep), TEMPLATE_JSON_BOOL_LENGTH);
 
   if (net->last_seen) {
     oonf_clock_toIntervalString(&_value_if_lastseen,
@@ -349,21 +396,18 @@ _initialize_interface_values(struct oonf_layer2_net *net) {
 /**
  * Initialize the value buffers for an array of layer2 data objects
  * @param template viewer template
- * @param dst array of destination buffers
  * @param data array of data objects
- * @param meta array of metadata description of objects
- * @param count number of objects in each array
  */
 static void
-_initialize_net_data_values(struct oonf_viewer_template *template,
-    struct isonumber_str *dst, struct oonf_layer2_data *data) {
+_initialize_if_data_values(struct oonf_viewer_template *template,
+    struct oonf_layer2_data *data) {
   size_t i;
 
-  memset(dst, 0, sizeof(*dst) * OONF_LAYER2_NET_COUNT);
+  memset(_value_if_data, 0, sizeof(_value_if_data));
 
   for (i=0; i<OONF_LAYER2_NET_COUNT; i++) {
     if (oonf_layer2_has_value(&data[i])) {
-      isonumber_from_s64(&dst[i], oonf_layer2_get_value(&data[i]),
+      isonumber_from_s64(&_value_if_data[i], oonf_layer2_get_value(&data[i]),
           oonf_layer2_get_net_metadata(i)->unit,
           oonf_layer2_get_net_metadata(i)->fraction,
           oonf_layer2_get_net_metadata(i)->binary,
@@ -373,27 +417,18 @@ _initialize_net_data_values(struct oonf_viewer_template *template,
 }
 
 /**
- * Initialize the value buffers for an array of layer2 data objects
- * @param template viewer template
- * @param dst array of destination buffers
+ * Initialize the network origin buffers for an array of layer2 data objects
  * @param data array of data objects
- * @param meta array of metadata description of objects
- * @param count number of objects in each array
  */
 static void
-_initialize_neigh_data_values(struct oonf_viewer_template *template,
-    struct isonumber_str *dst, struct oonf_layer2_data *data) {
+_initialize_if_origin_values(struct oonf_layer2_data *data) {
   size_t i;
 
-  memset(dst, 0, sizeof(*dst) * OONF_LAYER2_NEIGH_COUNT);
+  memset(_value_if_origin, 0, sizeof(_value_if_origin));
 
-  for (i=0; i<OONF_LAYER2_NEIGH_COUNT; i++) {
+  for (i=0; i<OONF_LAYER2_NET_COUNT; i++) {
     if (oonf_layer2_has_value(&data[i])) {
-      isonumber_from_s64(&dst[i], oonf_layer2_get_value(&data[i]),
-          oonf_layer2_get_neigh_metadata(i)->unit,
-          oonf_layer2_get_neigh_metadata(i)->fraction,
-          oonf_layer2_get_neigh_metadata(i)->binary,
-          template->create_raw);
+      strscpy(_value_if_origin[i], oonf_layer2_get_origin(&data[i])->name, IF_NAMESIZE);
     }
   }
 }
@@ -403,7 +438,7 @@ _initialize_neigh_data_values(struct oonf_viewer_template *template,
  * @param neigh layer2 neighbor
  */
 static void
-_initialize_neighbor_values(struct oonf_layer2_neigh *neigh) {
+_initialize_neigh_values(struct oonf_layer2_neigh *neigh) {
   netaddr_to_string(&_value_neigh_addr, &neigh->addr);
 
   if (neigh->last_seen) {
@@ -415,6 +450,47 @@ _initialize_neighbor_values(struct oonf_layer2_neigh *neigh) {
   }
 }
 
+/**
+ * Initialize the value buffers for an array of layer2 data objects
+ * @param template viewer template
+ * @param data array of data objects
+ */
+static void
+_initialize_neigh_data_values(struct oonf_viewer_template *template,
+    struct oonf_layer2_data *data) {
+  size_t i;
+
+  memset(_value_neigh_data, 0, sizeof(_value_neigh_data));
+
+  for (i=0; i<OONF_LAYER2_NEIGH_COUNT; i++) {
+    if (oonf_layer2_has_value(&data[i])) {
+      isonumber_from_s64(&_value_neigh_data[i],
+          oonf_layer2_get_value(&data[i]),
+          oonf_layer2_get_neigh_metadata(i)->unit,
+          oonf_layer2_get_neigh_metadata(i)->fraction,
+          oonf_layer2_get_neigh_metadata(i)->binary,
+          template->create_raw);
+    }
+  }
+}
+
+/**
+ * Initialize the network origin buffers for an array of layer2 data objects
+ * @param data array of data objects
+ */
+static void
+_initialize_neigh_origin_values(struct oonf_layer2_data *data) {
+  size_t i;
+
+  memset(_value_neigh_origin, 0, sizeof(_value_neigh_origin));
+
+  for (i=0; i<OONF_LAYER2_NEIGH_COUNT; i++) {
+    if (oonf_layer2_has_value(&data[i])) {
+      strscpy(_value_neigh_origin[i], oonf_layer2_get_origin(&data[i])->name, IF_NAMESIZE);
+    }
+  }
+}
+
 /**
  * Initialize the value buffers for a layer2 destination
  * @param l2dst layer2 destination
@@ -422,6 +498,7 @@ _initialize_neighbor_values(struct oonf_layer2_neigh *neigh) {
 static void
 _initialize_destination_values(struct oonf_layer2_destination *l2dst) {
   netaddr_to_string(&_value_dst_addr, &l2dst->destination);
+  strscpy(_value_dst_origin, l2dst->origin->name, IF_NAMESIZE);
 }
 
 /**
@@ -434,8 +511,9 @@ _cb_create_text_interface(struct oonf_viewer_template *template) {
   struct oonf_layer2_net *net;
 
   avl_for_each_element(oonf_layer2_get_network_tree(), net, _node) {
-    _initialize_interface_values(net);
-    _initialize_net_data_values(template, _value_if_data, net->data);
+    _initialize_if_values(net);
+    _initialize_if_data_values(template, net->data);
+    _initialize_if_origin_values(net->data);
 
     /* generate template output */
     oonf_viewer_output_print_line(template);
@@ -454,11 +532,12 @@ _cb_create_text_neighbor(struct oonf_viewer_template *template) {
   struct oonf_layer2_net *net;
 
   avl_for_each_element(oonf_layer2_get_network_tree(), net, _node) {
-    _initialize_interface_values(net);
+    _initialize_if_values(net);
 
     avl_for_each_element(&net->neighbors, neigh, _node) {
-      _initialize_neighbor_values(neigh);
-      _initialize_neigh_data_values(template, _value_neigh_data, neigh->data);
+      _initialize_neigh_values(neigh);
+      _initialize_neigh_data_values(template, neigh->data);
+      _initialize_neigh_origin_values(neigh->data);
 
       /* generate template output */
       oonf_viewer_output_print_line(template);
@@ -478,8 +557,9 @@ _cb_create_text_default(struct oonf_viewer_template *template) {
   struct oonf_layer2_net *net;
 
   avl_for_each_element(oonf_layer2_get_network_tree(), net, _node) {
-    _initialize_interface_values(net);
-    _initialize_neigh_data_values(template, _value_neigh_data, net->neighdata);
+    _initialize_if_values(net);
+    _initialize_neigh_data_values(template, net->neighdata);
+    _initialize_neigh_origin_values(net->neighdata);
 
     /* generate template output */
     oonf_viewer_output_print_line(template);
@@ -499,10 +579,10 @@ _cb_create_text_dst(struct oonf_viewer_template *template) {
   struct oonf_layer2_net *net;
 
   avl_for_each_element(oonf_layer2_get_network_tree(), net, _node) {
-    _initialize_interface_values(net);
+    _initialize_if_values(net);
 
     avl_for_each_element(&net->neighbors, neigh, _node) {
-      _initialize_neighbor_values(neigh);
+      _initialize_neigh_values(neigh);
 
       avl_for_each_element(&neigh->destinations, l2dst, _node) {
         _initialize_destination_values(l2dst);
