@@ -56,7 +56,6 @@
 #include "core/oonf_subsystem.h"
 #include "subsystems/oonf_class.h"
 #include "subsystems/oonf_rfc5444.h"
-#include "subsystems/oonf_timer.h"
 
 #include "nhdp/nhdp.h"
 #include "nhdp/nhdp_db.h"
@@ -71,13 +70,17 @@
 
 /* FIXME remove unneeded includes */
 
-/* definitions */
-#define LOG_MPR _nhdp_mpr_subsystem.logging
-
 /* prototypes */
+static void _early_cfg_init(void);
 static int _init(void);
 static void _cleanup(void);
-static void _cb_update_mpr(void);
+static void _cb_update_routing_mpr(struct nhdp_domain *);
+static void _cb_update_flooding_mpr(struct nhdp_domain *);
+
+#ifndef NDEBUG
+static void _validate_mpr_set(
+    const struct nhdp_domain *domain, struct neighbor_graph *graph);
+#endif
 
 static const char *_dependencies[] = {
   OONF_CLASS_SUBSYSTEM,
@@ -90,6 +93,7 @@ static struct oonf_subsystem _nhdp_mpr_subsystem = {
   .dependencies_count = ARRAYSIZE(_dependencies),
   .descr = "RFC7181 Appendix B MPR Plugin",
   .author = "Jonathan Kirchhoff",
+  .early_cfg_init = _early_cfg_init,
 
   .init = _init,
   .cleanup = _cleanup,
@@ -98,9 +102,21 @@ DECLARE_OONF_PLUGIN(_nhdp_mpr_subsystem);
 
 static struct nhdp_domain_mpr _mpr_handler = {
   .name = OONF_MPR_SUBSYSTEM,
-  .update_mpr = _cb_update_mpr,
+  .update_routing_mpr = _cb_update_routing_mpr,
+  .update_flooding_mpr = _cb_update_flooding_mpr,
 };
 
+/* logging sources for NHDP subsystem */
+enum oonf_log_source LOG_MPR;
+
+/**
+ * Initialize additional logging sources for NHDP
+ */
+static void
+_early_cfg_init(void) {
+  LOG_MPR = _nhdp_mpr_subsystem.logging;
+}
+
 /**
  * Initialize plugin
  * @return -1 if an error happened, 0 otherwise
@@ -122,188 +138,162 @@ _cleanup(void) {
 
 /**
  * Updates the current routing MPR selection in the NHDP database
- * @param current_mpr_data
+ * @param graph MPR neighbor graph instance
  */
 static void
-_update_nhdp_routing(struct neighbor_graph *graph) {
-  struct nhdp_link *lnk;
+_update_nhdp_routing(struct nhdp_domain *domain, struct neighbor_graph *graph) {
   struct n1_node *current_mpr_node;
-#ifdef OONF_LOG_DEBUG_INFO
-  struct netaddr_str buf1;
-#endif
-
-  OONF_DEBUG(LOG_MPR, "Updating ROUTING MPRs");
+  struct nhdp_link *lnk;
+  struct nhdp_neighbor_domaindata *neighbordata;
   
   list_for_each_element(nhdp_db_get_link_list(), lnk, _global_node) {
-    lnk->neigh->_domaindata[0].neigh_is_mpr = false;
+    neighbordata = nhdp_domain_get_neighbordata(domain, lnk->neigh);
+    neighbordata->neigh_is_mpr = false;
     current_mpr_node = avl_find_element(&graph->set_mpr,
         &lnk->neigh->originator,
         current_mpr_node, _avl_node);
-    if (current_mpr_node != NULL) {
-      OONF_DEBUG(LOG_MPR, "Processing MPR node %s",
-          netaddr_to_string(&buf1, &current_mpr_node->addr));
-      lnk->neigh->_domaindata[0].neigh_is_mpr = true;
-    }
+
+    neighbordata->neigh_is_mpr = current_mpr_node != NULL;
   }
 }
 
 /**
  * Updates the current flooding MPR selection in the NHDP database
- * @param current_mpr_data
+ * @param graph MPR neighbor graph instance
  */
 static void
 _update_nhdp_flooding(struct neighbor_graph *graph) {
   struct nhdp_link *current_link;
   struct n1_node *current_mpr_node;
-#ifdef OONF_LOG_DEBUG_INFO
-  struct netaddr_str buf1;
-#endif
-
-  OONF_DEBUG(LOG_MPR, "Updating FLOODING MPRs");
 
   list_for_each_element(nhdp_db_get_link_list(), current_link, _global_node) {
     current_mpr_node = avl_find_element(&graph->set_mpr,
         &current_link->neigh->originator,
         current_mpr_node, _avl_node);
-    if (current_mpr_node != NULL) {
-      OONF_DEBUG(LOG_MPR, "Processing MPR node %s",
-          netaddr_to_string(&buf1, &current_mpr_node->addr));
-      current_link->neigh->neigh_is_flooding_mpr = true;
-    }
+
+    current_link->neigh_is_flooding_mpr = current_mpr_node != NULL;
   }
 }
 
 /**
  * Updates the current flooding MPR selection in the NHDP database
- * @param current_mpr_data
  */
 static void
 _clear_nhdp_flooding(void) {
   struct nhdp_link *current_link;
 
-  OONF_DEBUG(LOG_MPR, "Updating FLOODING MPRs");
+//  OONF_DEBUG(LOG_MPR, "Updating FLOODING MPRs");
 
   list_for_each_element(nhdp_db_get_link_list(), current_link, _global_node) {
-    current_link->neigh->neigh_is_flooding_mpr = false;
+    current_link->neigh_is_flooding_mpr = false;
   }
 }
 
+/**
+ * Update the flooding MPR settings 
+ */
 static void
-_update_flooding_mpr(void) {
+_cb_update_flooding_mpr(struct nhdp_domain *domain) {
   struct mpr_flooding_data flooding_data;
 
   memset(&flooding_data, 0, sizeof(flooding_data));
   
-  if (nhdp_domain_get_flooding()->mpr != &_mpr_handler) {
-    /* we are not the flooding mpr */
-    return;
-  }
-
-  /* FIXME Currently, the flooding set is calculated incrementally (i.e. 
-   in a coordinated way as suggested by RFC 7181; however, this should
-   be configurable (and other selection algorithms might not be compatible
-   with this approach).
-   */
-  /* FIXME How to support the coordination flooding and routing MPRs 
-   * selection? */
-  /* calculate flooding MPRs */
   _clear_nhdp_flooding();
   avl_for_each_element(nhdp_interface_get_tree(), flooding_data.current_interface, _node) {
-    OONF_DEBUG(LOG_MPR, "Calculating flooding MPRs for interface %s",
+    OONF_DEBUG(LOG_MPR, "*** Calculate flooding MPRs for interface %s ***",
         nhdp_interface_get_name(flooding_data.current_interface));
     
-    mpr_calculate_neighbor_graph_flooding(
-        nhdp_domain_get_flooding(), &flooding_data);
-    mpr_calculate_mpr_rfc7181(nhdp_domain_get_flooding(),
-        &flooding_data.neigh_graph);
-    mpr_print_sets(&flooding_data.neigh_graph);
+    mpr_calculate_neighbor_graph_flooding(domain, &flooding_data);
+    mpr_calculate_mpr_rfc7181(domain, &flooding_data.neigh_graph);
+    mpr_print_sets(domain, &flooding_data.neigh_graph);
+#ifndef NDEBUG
+    _validate_mpr_set(domain, &flooding_data.neigh_graph);
+#endif
     _update_nhdp_flooding(&flooding_data.neigh_graph);
+    mpr_clear_neighbor_graph(&flooding_data.neigh_graph);
   }
 
-  /* free memory */
-  mpr_clear_neighbor_graph(&flooding_data.neigh_graph);
-}
-
-static void
-_update_routing_mpr(void) {
-  struct neighbor_graph routing_graph;
-  struct nhdp_domain *domain;
-
-  list_for_each_element(nhdp_domain_get_list(), domain, _node) {
-    if (domain->mpr != &_mpr_handler) {
-      /* we are not the routing MPR for this domain */
-      continue;
-    }
-    memset(&routing_graph, 0, sizeof(routing_graph));
-
-    mpr_calculate_neighbor_graph_routing(domain, &routing_graph);
-    mpr_calculate_mpr_rfc7181(domain, &routing_graph);
-    mpr_print_sets(&routing_graph);
-    _update_nhdp_routing(&routing_graph);
-  }
 }
 
 /**
- * Callback triggered when an MPR update is required
+ * Update the routing MPR settings for all domains
  */
 static void
-_cb_update_mpr(void) {
-  OONF_DEBUG(LOG_MPR, "Recalculating MPRs");
+_cb_update_routing_mpr(struct nhdp_domain *domain) {
+  struct neighbor_graph routing_graph;
 
-  /* calculate flooding MPRs */
-  _update_flooding_mpr();
-  
-  /* calculate routing MPRs */
-  _update_routing_mpr();
-
-  OONF_DEBUG(LOG_MPR, "Finished recalculating MPRs");
+  if (domain->mpr != &_mpr_handler) {
+    /* we are not the routing MPR for this domain */
+    return;
+  }
+  OONF_DEBUG(LOG_MPR, "*** Calculate routing MPRs for domain %u ***", domain->index);
+    
+  memset(&routing_graph, 0, sizeof(routing_graph));
+  mpr_calculate_neighbor_graph_routing(domain, &routing_graph);
+  mpr_calculate_mpr_rfc7181(domain, &routing_graph);
+  mpr_print_sets(domain, &routing_graph);
+#ifndef NDEBUG
+  _validate_mpr_set(domain, &routing_graph);
+#endif
+  _update_nhdp_routing(domain, &routing_graph);
+  mpr_clear_neighbor_graph(&routing_graph);
 }
 
-#if 0
+#ifndef NDEBUG
 
 /**
  * Validate the MPR set according to section 18.3 (draft 19)
- * @param current_mpr_data
- * @return 
+ * @param domain NHDP domaine
+ * @param graph MPR neighbor graph instance
  */
-void
-_validate_mpr_set(struct common_data *mpr_data) {
+static void
+_validate_mpr_set(const struct nhdp_domain *domain, struct neighbor_graph *graph)
+{
   struct n1_node *node_n1;
   struct addr_node *n2_addr;
-  uint32_t d_y_n1, d_y_mpr;
+  uint32_t d_y_n1;
+  uint32_t d_y_mpr;
 
   OONF_DEBUG(LOG_MPR, "Validating MPR set");
 
   /* 
    * First property: If x in N1 has W(x) = WILL_ALWAYS then x is in M. 
    */
-  avl_for_each_element(&mpr_data->set_n1, node_n1,
-      _avl_node) {
-    if (node_n1->neigh->flooding_willingness
-        == RFC5444_WILLINGNESS_ALWAYS) {
-      assert(_is_mpr(mpr_data, &node_n1->addr));
+  avl_for_each_element(&graph->set_n1, node_n1, _avl_node) {
+    if (domain == nhdp_domain_get_flooding_domain()) {
+      if (node_n1->link->flooding_willingness
+            == RFC7181_WILLINGNESS_ALWAYS) {
+        assert(mpr_is_mpr(graph, &node_n1->addr));
+      }
+    }
+    else {
+      struct nhdp_neighbor_domaindata *neighdata;
+
+      neighdata = nhdp_domain_get_neighbordata(domain, node_n1->neigh);
+      if (neighdata->willingness == RFC7181_WILLINGNESS_ALWAYS) {
+        assert(mpr_is_mpr(graph, &node_n1->addr));
+      }
     }
   }
 
-  /*
-   * Second property: For any y in N2 that does not have a defined d1(y), 
-   * there is at least one element in M that is also in N1(y). This is 
-   * equivalent to the requirement that d(y, M) is defined.
-   */
-  avl_for_each_element(&mpr_data->set_n2, n2_addr, _avl_node) {
-    assert(_calculate_d_of_y_s(mpr_data, n2_addr, &mpr_data->set_mpr)
-        != RFC5444_METRIC_INFINITE);
-  }
+  avl_for_each_element(&graph->set_n2, n2_addr, _avl_node) {
+    d_y_n1 = mpr_calculate_d_of_y_s(domain, graph, n2_addr, &graph->set_n1);
+    d_y_mpr = mpr_calculate_d_of_y_s(domain, graph, n2_addr, &graph->set_mpr);
+    
+    OONF_DEBUG(LOG_MPR, "d_y_n1 = %u", d_y_n1);
+    OONF_DEBUG(LOG_MPR, "d_y_mpr = %u", d_y_mpr);
 
-  /*
-   * Third property: For any y in N2, d(y,M) = d(y, N1).
-   */
-  avl_for_each_element(&mpr_data->set_n2, n2_addr, _avl_node) {
-    d_y_n1 = _calculate_d_of_y_s(mpr_data, n2_addr, &mpr_data->set_n1);
-    d_y_mpr = _calculate_d_of_y_s(mpr_data, n2_addr, &mpr_data->set_mpr);
-    assert(d_y_n1 == d_y_mpr);
+    /*
+     * Second property: For any y in N2 that does not have a defined d1(y), 
+     * there is at least one element in M that is also in N1(y). This is 
+     * equivalent to the requirement that d(y, M) is defined.
+     */
+    assert(d_y_mpr < RFC7181_METRIC_INFINITE_PATH);
+
+    /*
+     * Third property: For any y in N2, d(y,M) = d(y, N1).
+     */
+    assert(d_y_mpr == d_y_n1);
   }
 }
-
-
 #endif
