@@ -18,7 +18,7 @@ EXPORT int os_fd_generic_join_mcast_recv(struct os_fd *sock,
 
 EXPORT int os_fd_generic_join_mcast_send(struct os_fd *sock,
     const struct netaddr *multicast,
-    const struct os_interface *os_if, bool loop,
+    const struct os_interface *os_if, bool loop, uint8_t ttl,
     enum oonf_log_source log_src);
 
 #endif /* _OS_FD_GENERIC_JOIN_MCAST_H_ */
