@@ -109,7 +109,8 @@ static void _early_cfg_init(void);
 static struct os_interface *_add_interface(const char *name);
 static void _remove_interface(struct os_interface *data);
 
-static int _init_mesh(struct os_interface *os_if);
+static void _init_mesh(struct os_interface *os_if);
+static void _refresh_mesh(struct os_interface *os_if, char *old_redirect, char *old_spoof);
 static void _cleanup_mesh(struct os_interface *os_if);
 
 static void _query_interface_links(void);
@@ -271,13 +272,15 @@ static void
 _cleanup(void) {
   struct os_interface_listener *if_listener, *if_listener_it;
   struct os_interface *os_if, *os_if_it;
+  bool configured;
 
   avl_for_each_element_safe(&_interface_data_tree, os_if, _node, os_if_it) {
+    configured = os_if->_internal.configured;
     list_for_each_element_safe(&os_if->_listeners, if_listener, _node, if_listener_it) {
       os_interface_linux_remove(if_listener);
     }
 
-    if (os_if->_internal.configured) {
+    if (configured) {
       os_if->_internal.configured = false;
       _remove_interface(os_if);
     }
@@ -327,7 +330,7 @@ os_interface_linux_add(struct os_interface_listener *if_listener) {
   list_add_tail(&data->_listeners, &if_listener->_node);
 
   if (if_listener->mesh && if_listener->name != _ANY_INTERFACE) {
-    if (!data->_internal.mesh_counter
+    if (data->_internal.mesh_counter == 0
         && !data->_internal.ignore_mesh) {
       _init_mesh(data);
     }
@@ -567,6 +570,7 @@ _add_interface(const char *name) {
 
     /* initialize list/tree */
     avl_init(&data->addresses, avl_comp_netaddr, false);
+    avl_init(&data->peers, avl_comp_netaddr, false);
     list_init_head(&data->_listeners);
 
     /* initialize change timer */
@@ -575,12 +579,17 @@ _add_interface(const char *name) {
     /* check if this is the unspecified interface "any" */
     if (strcmp(data->name, _ANY_INTERFACE) == 0) {
       data->flags.any = true;
+      data->flags.up = true;
     }
 
     /* trigger new queries */
     _trigger_link_query = true;
     _trigger_address_query = true;
 
+    data->if_linklocal_v4 = &NETADDR_UNSPEC;
+    data->if_linklocal_v6 = &NETADDR_UNSPEC;
+    data->if_v4 = &NETADDR_UNSPEC;
+    data->if_v6 = &NETADDR_UNSPEC;
     _query_interface_links();
   }
 
@@ -599,7 +608,7 @@ _remove_interface(struct os_interface *data) {
     return;
   }
 
-  if (data->_internal.mesh_settings_active) {
+  if (data->flags.mesh) {
     _cleanup_mesh(data);
   }
 
@@ -608,6 +617,10 @@ _remove_interface(struct os_interface *data) {
     avl_remove(&data->addresses, &ip->_node);
     oonf_class_free(&_interface_ip_class, ip);
   }
+  avl_for_each_element_safe(&data->peers, ip, _node, ip_iter) {
+    avl_remove(&data->peers, &ip->_node);
+    oonf_class_free(&_interface_ip_class, ip);
+  }
 
   /* stop change timer */
   oonf_timer_stop(&data->_change_timer);
@@ -620,23 +633,21 @@ _remove_interface(struct os_interface *data) {
 /**
  * Initialize interface for mesh usage
  * @param os_if network interface data
- * @return -1 if an error happened, 0 otherwise
  */
-static int
+static void
 _init_mesh(struct os_interface *os_if) {
-  char procfile[FILENAME_MAX];
-  char old_redirect = 0, old_spoof = 0;
-
   if (os_if->flags.loopback || os_if->flags.any) {
     /* ignore loopback and unspecific interface*/
-    return 0;
+    return;
   }
 
-  if (os_if->_internal.mesh_settings_active) {
+  if (os_if->flags.mesh) {
     /* mesh settings already active or not used for this interface */
-    return 0;
+    return;
   }
-  os_if->_internal.mesh_settings_active = true;
+  os_if->flags.mesh = true;
+
+  OONF_DEBUG(LOG_OS_INTERFACE, "Init mesh: %s", os_if->name);
 
   /* handle global ip_forward setting */
   _mesh_count++;
@@ -644,10 +655,30 @@ _init_mesh(struct os_interface *os_if) {
     _activate_if_routing();
   }
 
+  _refresh_mesh(os_if,
+      &os_if->_internal._original_icmp_redirect,
+      &os_if->_internal._original_ip_spoof);
+}
+
+static void
+_refresh_mesh(struct os_interface *os_if, char *old_redirect, char *old_spoof) {
+  char procfile[FILENAME_MAX];
+  if (os_if->flags.loopback || os_if->flags.any) {
+    /* ignore loopback and unspecific interface*/
+    return;
+  }
+
+  if (!os_if->flags.mesh) {
+    /* this is no mesh interface */
+    return;
+  }
+
+  OONF_DEBUG(LOG_OS_INTERFACE, "Refresh mesh: %s", os_if->name);
+
   /* Generate the procfile name */
   snprintf(procfile, sizeof(procfile), PROC_IF_REDIRECT, os_if->name);
 
-  if (_os_linux_writeToFile(procfile, &old_redirect, '0')) {
+  if (_os_linux_writeToFile(procfile, old_redirect, '0')) {
     OONF_WARN(LOG_OS_INTERFACE, "WARNING! Could not disable ICMP redirects! "
         "You should manually ensure that ICMP redirects are disabled!");
   }
@@ -655,12 +686,176 @@ _init_mesh(struct os_interface *os_if) {
   /* Generate the procfile name */
   snprintf(procfile, sizeof(procfile), PROC_IF_SPOOF, os_if->name);
 
-  if (_os_linux_writeToFile(procfile, &old_spoof, '0')) {
+  if (_os_linux_writeToFile(procfile, old_spoof, '0')) {
     OONF_WARN(LOG_OS_INTERFACE, "WARNING! Could not disable the IP spoof filter! "
         "You should mannually ensure that IP spoof filtering is disabled!");
   }
+}
+
+/**
+ * Cleanup interface after mesh usage
+ * @param os_if network interface
+ */
+static void
+_cleanup_mesh(struct os_interface *os_if) {
+  char procfile[FILENAME_MAX];
+
+  if (os_if->flags.loopback || os_if->flags.any) {
+    /* ignore loopback and unspecific interface*/
+    return;
+  }
+
+  if (!os_if->flags.mesh) {
+    /* mesh settings not active */
+    return;
+  }
+
+  OONF_DEBUG(LOG_OS_INTERFACE, "Cleanup mesh: %s", os_if->name);
+
+  /* Generate the procfile name */
+  snprintf(procfile, sizeof(procfile), PROC_IF_REDIRECT, os_if->name);
+
+  if (_os_linux_writeToFile(procfile, NULL,
+      os_if->_internal._original_icmp_redirect) != 0) {
+    OONF_WARN(LOG_OS_INTERFACE, "Could not restore ICMP redirect flag %s to %c",
+        procfile, os_if->_internal._original_icmp_redirect);
+  }
+
+  /* Generate the procfile name */
+  snprintf(procfile, sizeof(procfile), PROC_IF_SPOOF, os_if->name);
+
+  if (_os_linux_writeToFile(procfile, NULL,
+      os_if->_internal._original_ip_spoof) != 0) {
+    OONF_WARN(LOG_OS_INTERFACE, "Could not restore IP spoof flag %s to %c",
+        procfile, os_if->_internal._original_ip_spoof);
+  }
+
+  /* handle global ip_forward setting */
+  _mesh_count--;
+  if (_mesh_count == 0) {
+    _deactivate_if_routing();
+  }
+
+  return;
+}
+
+/**
+ * Set the required settings to allow multihop mesh routing
+ */
+static void
+_activate_if_routing(void) {
+  if (_os_linux_writeToFile(PROC_IPFORWARD_V4, &_original_ipv4_forward, '1')) {
+    OONF_WARN(LOG_OS_INTERFACE, "WARNING! Could not activate ip_forward for ipv4! "
+        "You should manually ensure that ip_forward for ipv4 is activated!");
+  }
+  if (os_system_is_ipv6_supported()) {
+    if(_os_linux_writeToFile(PROC_IPFORWARD_V6, &_original_ipv6_forward, '1')) {
+      OONF_WARN(LOG_OS_INTERFACE, "WARNING! Could not activate ip_forward for ipv6! "
+          "You should manually ensure that ip_forward for ipv6 is activated!");
+    }
+  }
+
+  if (_os_linux_writeToFile(PROC_ALL_REDIRECT, &_original_icmp_redirect, '0')) {
+    OONF_WARN(LOG_OS_INTERFACE, "WARNING! Could not disable ICMP redirects! "
+        "You should manually ensure that ICMP redirects are disabled!");
+  }
+
+  /* check kernel version and disable global rp_filter */
+  if (_is_kernel_2_6_31_or_better) {
+    if (_os_linux_writeToFile(PROC_ALL_SPOOF, &_original_rp_filter, '0')) {
+      OONF_WARN(LOG_OS_INTERFACE, "WARNING! Could not disable global rp_filter "
+          "(necessary for kernel 2.6.31 and newer)! You should manually "
+          "ensure that rp_filter is disabled!");
+    }
+  }
+}
+
+/**
+ * Reset the multihop mesh routing settings to default
+ */
+static void
+_deactivate_if_routing(void) {
+  if (_os_linux_writeToFile(PROC_ALL_REDIRECT, NULL, _original_icmp_redirect) != 0) {
+    OONF_WARN(LOG_OS_INTERFACE,
+        "WARNING! Could not restore ICMP redirect flag %s to %c!",
+        PROC_ALL_REDIRECT, _original_icmp_redirect);
+  }
+
+  if (_os_linux_writeToFile(PROC_ALL_SPOOF, NULL, _original_rp_filter)) {
+    OONF_WARN(LOG_OS_INTERFACE,
+        "WARNING! Could not restore global rp_filter flag %s to %c!",
+        PROC_ALL_SPOOF, _original_rp_filter);
+  }
+
+  if (_os_linux_writeToFile(PROC_IPFORWARD_V4, NULL, _original_ipv4_forward)) {
+    OONF_WARN(LOG_OS_INTERFACE, "WARNING! Could not restore %s to %c!",
+        PROC_IPFORWARD_V4, _original_ipv4_forward);
+  }
+  if (os_system_is_ipv6_supported()) {
+    if (_os_linux_writeToFile(PROC_IPFORWARD_V6, NULL, _original_ipv6_forward)) {
+      OONF_WARN(LOG_OS_INTERFACE, "WARNING! Could not restore %s to %c",
+          PROC_IPFORWARD_V6, _original_ipv6_forward);
+    }
+  }
+}
+
+/**
+ * Overwrite a numeric entry in the procfile system and keep the old
+ * value.
+ * @param file pointer to filename (including full path)
+ * @param old pointer to memory to store old value
+ * @param value new value
+ * @return -1 if an error happened, 0 otherwise
+ */
+static int
+_os_linux_writeToFile(const char *file, char *old, char value) {
+  int fd;
+  char rv;
+
+  if (value == 0) {
+    /* ignore */
+    return 0;
+  }
+
+  if ((fd = open(file, O_RDWR)) < 0) {
+    OONF_WARN(LOG_OS_INTERFACE,
+      "Error, cannot open proc entry %s: %s (%d)\n",
+      file, strerror(errno), errno);
+    return -1;
+  }
+
+  if (read(fd, &rv, 1) != 1) {
+    OONF_WARN(LOG_OS_INTERFACE,
+      "Error, cannot read proc entry %s: %s (%d)\n",
+      file, strerror(errno), errno);
+    close(fd);
+    return -1;
+  }
+
+  if (rv != value) {
+    if (lseek(fd, SEEK_SET, 0) == -1) {
+      OONF_WARN(LOG_OS_INTERFACE,
+        "Error, cannot rewind to start on proc entry %s: %s (%d)\n",
+        file, strerror(errno), errno);
+      close(fd);
+      return -1;
+    }
+
+    if (write(fd, &value, 1) != 1) {
+      OONF_WARN(LOG_OS_INTERFACE,
+        "Error, cannot write '%c' to proc entry %s: %s (%d)\n",
+        value, file, strerror(errno), errno);
+    }
+
+    OONF_DEBUG(LOG_OS_INTERFACE, "Writing '%c' (was %c) to %s", value, rv, file);
+  }
+
+  close(fd);
+
+  if (old && rv != value) {
+    *old = rv;
+  }
 
-  os_if->_internal._original_state = (old_redirect << 8) | (old_spoof);
   return 0;
 }
 
@@ -736,174 +931,6 @@ _query_interface_addresses(void) {
   os_system_linux_netlink_send(&_rtnetlink_if_query, msg);
 }
 
-/**
- * Cleanup interface after mesh usage
- * @param data network interface data
- */
-static void
-_cleanup_mesh(struct os_interface *os_if) {
-  char restore_redirect, restore_spoof;
-  char procfile[FILENAME_MAX];
-
-  if (os_if->flags.loopback || os_if->flags.any) {
-    /* ignore loopback and unspecific interface*/
-    return;
-  }
-
-  if (!os_if->_internal.mesh_settings_active) {
-    /* mesh settings not active */
-    return;
-  }
-  restore_redirect = (os_if->_internal._original_state >> 8) & 255;
-  restore_spoof = (os_if->_internal._original_state & 255);
-
-  /* Generate the procfile name */
-  snprintf(procfile, sizeof(procfile), PROC_IF_REDIRECT, os_if->name);
-
-  if (_os_linux_writeToFile(procfile, NULL, restore_redirect) != 0) {
-    OONF_WARN(LOG_OS_INTERFACE, "Could not restore ICMP redirect flag %s to %c",
-        procfile, restore_redirect);
-  }
-
-  /* Generate the procfile name */
-  snprintf(procfile, sizeof(procfile), PROC_IF_SPOOF, os_if->name);
-
-  if (_os_linux_writeToFile(procfile, NULL, restore_spoof) != 0) {
-    OONF_WARN(LOG_OS_INTERFACE, "Could not restore IP spoof flag %s to %c",
-        procfile, restore_spoof);
-  }
-
-  /* handle global ip_forward setting */
-  _mesh_count--;
-  if (_mesh_count == 0) {
-    _deactivate_if_routing();
-  }
-
-  os_if->_internal._original_state = 0;
-  return;
-}
-
-/**
- * Set the required settings to allow multihop mesh routing
- */
-static void
-_activate_if_routing(void) {
-  if (_os_linux_writeToFile(PROC_IPFORWARD_V4, &_original_ipv4_forward, '1')) {
-    OONF_WARN(LOG_OS_INTERFACE, "WARNING! Could not activate ip_forward for ipv4! "
-        "You should manually ensure that ip_forward for ipv4 is activated!");
-  }
-  if (os_system_is_ipv6_supported()) {
-    if(_os_linux_writeToFile(PROC_IPFORWARD_V6, &_original_ipv6_forward, '1')) {
-      OONF_WARN(LOG_OS_INTERFACE, "WARNING! Could not activate ip_forward for ipv6! "
-          "You should manually ensure that ip_forward for ipv6 is activated!");
-    }
-  }
-
-  if (_os_linux_writeToFile(PROC_ALL_REDIRECT, &_original_icmp_redirect, '0')) {
-    OONF_WARN(LOG_OS_INTERFACE, "WARNING! Could not disable ICMP redirects! "
-        "You should manually ensure that ICMP redirects are disabled!");
-  }
-
-  /* check kernel version and disable global rp_filter */
-  if (_is_kernel_2_6_31_or_better) {
-    if (_os_linux_writeToFile(PROC_ALL_SPOOF, &_original_rp_filter, '0')) {
-      OONF_WARN(LOG_OS_INTERFACE, "WARNING! Could not disable global rp_filter "
-          "(necessary for kernel 2.6.31 and newer)! You should manually "
-          "ensure that rp_filter is disabled!");
-    }
-  }
-}
-
-/**
- * Reset the multihop mesh routing settings to default
- */
-static void
-_deactivate_if_routing(void) {
-  if (_os_linux_writeToFile(PROC_ALL_REDIRECT, NULL, _original_icmp_redirect) != 0) {
-    OONF_WARN(LOG_OS_INTERFACE,
-        "WARNING! Could not restore ICMP redirect flag %s to %c!",
-        PROC_ALL_REDIRECT, _original_icmp_redirect);
-  }
-
-  if (_os_linux_writeToFile(PROC_ALL_SPOOF, NULL, _original_rp_filter)) {
-    OONF_WARN(LOG_OS_INTERFACE,
-        "WARNING! Could not restore global rp_filter flag %s to %c!",
-        PROC_ALL_SPOOF, _original_rp_filter);
-  }
-
-  if (_os_linux_writeToFile(PROC_IPFORWARD_V4, NULL, _original_ipv4_forward)) {
-    OONF_WARN(LOG_OS_INTERFACE, "WARNING! Could not restore %s to %c!",
-        PROC_IPFORWARD_V4, _original_ipv4_forward);
-  }
-  if (os_system_is_ipv6_supported()) {
-    if (_os_linux_writeToFile(PROC_IPFORWARD_V6, NULL, _original_ipv6_forward)) {
-      OONF_WARN(LOG_OS_INTERFACE, "WARNING! Could not restore %s to %c",
-          PROC_IPFORWARD_V6, _original_ipv6_forward);
-    }
-  }
-}
-
-
-/**
- * Overwrite a numeric entry in the procfile system and keep the old
- * value.
- * @param file pointer to filename (including full path)
- * @param old pointer to memory to store old value
- * @param value new value
- * @return -1 if an error happened, 0 otherwise
- */
-static int
-_os_linux_writeToFile(const char *file, char *old, char value) {
-  int fd;
-  char rv;
-
-  if (value == 0) {
-    /* ignore */
-    return 0;
-  }
-
-  if ((fd = open(file, O_RDWR)) < 0) {
-    OONF_WARN(LOG_OS_INTERFACE,
-      "Error, cannot open proc entry %s: %s (%d)\n",
-      file, strerror(errno), errno);
-    return -1;
-  }
-
-  if (read(fd, &rv, 1) != 1) {
-    OONF_WARN(LOG_OS_INTERFACE,
-      "Error, cannot read proc entry %s: %s (%d)\n",
-      file, strerror(errno), errno);
-    close(fd);
-    return -1;
-  }
-
-  if (rv != value) {
-    if (lseek(fd, SEEK_SET, 0) == -1) {
-      OONF_WARN(LOG_OS_INTERFACE,
-        "Error, cannot rewind to start on proc entry %s: %s (%d)\n",
-        file, strerror(errno), errno);
-      close(fd);
-      return -1;
-    }
-
-    if (write(fd, &value, 1) != 1) {
-      OONF_WARN(LOG_OS_INTERFACE,
-        "Error, cannot write '%c' to proc entry %s: %s (%d)\n",
-        value, file, strerror(errno), errno);
-    }
-
-    OONF_DEBUG(LOG_OS_INTERFACE, "Writing '%c' (was %c) to %s", value, rv, file);
-  }
-
-  close(fd);
-
-  if (old && rv != value) {
-    *old = rv;
-  }
-
-  return 0;
-}
-
 /**
  * Trigger all change listeners of a network interface
  * @param os_if network interface
@@ -952,6 +979,7 @@ _link_parse_nlmsg(const char *ifname, struct nlmsghdr *msg) {
   struct netaddr addr;
   struct os_interface *ifdata;
   int iflink;
+  bool old_up;
 #if defined(OONF_LOG_DEBUG_INFO)
   struct netaddr_str nbuf;
 #endif
@@ -965,6 +993,7 @@ _link_parse_nlmsg(const char *ifname, struct nlmsghdr *msg) {
     return;
   }
 
+  old_up = ifdata->flags.up;
   ifdata->flags.up = (ifi_msg->ifi_flags & IFF_UP) != 0;
   ifdata->flags.promisc = (ifi_msg->ifi_flags & IFF_PROMISC) != 0;
   ifdata->flags.pointtopoint = (ifi_msg->ifi_flags & IFF_POINTOPOINT) != 0;
@@ -982,6 +1011,10 @@ _link_parse_nlmsg(const char *ifname, struct nlmsghdr *msg) {
   ifdata->index = ifi_msg->ifi_index;
   ifdata->base_index = ifdata->index;
 
+  if (!old_up && ifdata->flags.up && ifdata->flags.mesh && !ifdata->_internal.ignore_mesh) {
+    /* refresh mesh parameters, might be gone for LTE-sticks */
+    _refresh_mesh(ifdata, NULL, NULL);
+  }
   for(; RTA_OK(ifi_attr, ifi_len); ifi_attr = RTA_NEXT(ifi_attr,ifi_len)) {
     switch(ifi_attr->rta_type) {
       case IFLA_ADDRESS:
@@ -1075,15 +1108,19 @@ _update_address_shortcuts(struct os_interface *os_if) {
  * Add an IP address/prefix to a network interface
  * @param os_if network interface
  * @param prefixed_addr full IP address with prefix length
+ * @param peer true if this is a peer address, false otherwise
  */
 static void
-_add_address(struct os_interface *os_if, struct netaddr *prefixed_addr) {
+_add_address(struct os_interface *os_if, struct netaddr *prefixed_addr, bool peer) {
   struct os_interface_ip *ip;
+  struct avl_tree *tree;
 #if defined(OONF_LOG_INFO)
   struct netaddr_str nbuf;
 #endif
 
-  ip = avl_find_element(&os_if->addresses, prefixed_addr, ip, _node);
+  tree = peer ? &os_if->peers : &os_if->addresses;
+
+  ip = avl_find_element(tree, prefixed_addr, ip, _node);
   if (!ip) {
     ip = oonf_class_malloc(&_interface_ip_class);
     if (!ip) {
@@ -1093,14 +1130,15 @@ _add_address(struct os_interface *os_if, struct netaddr *prefixed_addr) {
     /* establish key and add to tree */
     memcpy(&ip->prefixed_addr, prefixed_addr, sizeof(*prefixed_addr));
     ip->_node.key = &ip->prefixed_addr;
-    avl_insert(&os_if->addresses, &ip->_node);
+    avl_insert(tree, &ip->_node);
 
     /* add back pointer */
     ip->interf = os_if;
   }
 
-  OONF_INFO(LOG_OS_INTERFACE, "Add address to %s: %s",
-      os_if->name, netaddr_to_string(&nbuf, prefixed_addr));
+  OONF_INFO(LOG_OS_INTERFACE, "Add address to %s%s: %s",
+      os_if->name, peer ? " (peer)" : "",
+      netaddr_to_string(&nbuf, prefixed_addr));
 
   /* copy sanitized addresses */
   memcpy(&ip->address, prefixed_addr, sizeof(*prefixed_addr));
@@ -1112,23 +1150,27 @@ _add_address(struct os_interface *os_if, struct netaddr *prefixed_addr) {
  * Remove an IP address/prefix from a network interface
  * @param os_if network interface
  * @param prefixed_addr full IP address with prefix length
+ * @param peer true if this is a peer address, false otherwise
  */
 static void
-_remove_address(struct os_interface *os_if, struct netaddr *prefixed_addr) {
+_remove_address(struct os_interface *os_if, struct netaddr *prefixed_addr, bool peer) {
   struct os_interface_ip *ip;
+  struct avl_tree *tree;
 #if defined(OONF_LOG_INFO)
   struct netaddr_str nbuf;
 #endif
 
-  ip = avl_find_element(&os_if->addresses, prefixed_addr, ip, _node);
+  tree = peer ? &os_if->peers : &os_if->addresses;
+  ip = avl_find_element(tree, prefixed_addr, ip, _node);
   if (!ip) {
     return;
   }
 
-  OONF_INFO(LOG_OS_INTERFACE, "Remove address from %s: %s",
-      os_if->name, netaddr_to_string(&nbuf, prefixed_addr));
+  OONF_INFO(LOG_OS_INTERFACE, "Remove address from %s%s: %s",
+      os_if->name, peer ? " (peer)" : "",
+      netaddr_to_string(&nbuf, prefixed_addr));
 
-  avl_remove(&os_if->addresses, &ip->_node);
+  avl_remove(tree, &ip->_node);
   oonf_class_free(&_interface_ip_class, ip);
 }
 
@@ -1143,7 +1185,7 @@ _address_parse_nlmsg(const char *ifname, struct nlmsghdr *msg) {
   struct rtattr *ifa_attr;
   int ifa_len;
   struct os_interface *ifdata;
-  struct netaddr addr;
+  struct netaddr ifa_local, ifa_address;
   bool update;
 
   ifa_msg = NLMSG_DATA(msg);
@@ -1159,18 +1201,26 @@ _address_parse_nlmsg(const char *ifname, struct nlmsghdr *msg) {
       ifname, ifa_msg->ifa_index, ifa_len);
 
   update = false;
+  netaddr_invalidate(&ifa_local);
+  netaddr_invalidate(&ifa_address);
+
   for(; RTA_OK(ifa_attr, ifa_len); ifa_attr = RTA_NEXT(ifa_attr,ifa_len)) {
     switch(ifa_attr->rta_type) {
       case IFA_ADDRESS:
-        netaddr_from_binary_prefix(&addr, RTA_DATA(ifa_attr), RTA_PAYLOAD(ifa_attr), 0,
-            ifa_msg->ifa_prefixlen);
-        if (msg->nlmsg_type == RTM_NEWADDR) {
-          _add_address(ifdata, &addr);
+        netaddr_from_binary_prefix(&ifa_address,
+            RTA_DATA(ifa_attr), RTA_PAYLOAD(ifa_attr), 0,
+                ifa_msg->ifa_prefixlen);
+        if (netaddr_is_unspec(&ifa_local)) {
+          memcpy(&ifa_local, &ifa_address, sizeof(ifa_local));
         }
-        else {
-          _remove_address(ifdata, &addr);
+        break;
+      case IFA_LOCAL:
+        netaddr_from_binary_prefix(&ifa_local,
+            RTA_DATA(ifa_attr), RTA_PAYLOAD(ifa_attr), 0,
+                ifa_msg->ifa_prefixlen);
+        if (netaddr_is_unspec(&ifa_address)) {
+          memcpy(&ifa_address, &ifa_local, sizeof(ifa_address));
         }
-        update = true;
         break;
       default:
         OONF_DEBUG(LOG_OS_INTERFACE, "ifa_attr_type: %u", ifa_attr->rta_type);
@@ -1178,16 +1228,37 @@ _address_parse_nlmsg(const char *ifname, struct nlmsghdr *msg) {
     }
   }
 
-  if (update) {
+  if (!netaddr_is_unspec(&ifa_local)) {
+    if (msg->nlmsg_type == RTM_NEWADDR) {
+      _add_address(ifdata, &ifa_local, false);
+    }
+    else {
+      _remove_address(ifdata, &ifa_local, false);
+    }
+
     _update_address_shortcuts(ifdata);
+    update = true;
+  }
+
+  if (netaddr_cmp(&ifa_local, &ifa_address)) {
+    if (msg->nlmsg_type == RTM_NEWADDR) {
+      _add_address(ifdata, &ifa_address, true);
+    }
+    else {
+      _remove_address(ifdata, &ifa_address, true);
+    }
+
+    update = true;
+  }
+
+  if (update) {
     if (!ifdata->_addr_initialized) {
       ifdata->_addr_initialized = true;
       OONF_INFO(LOG_OS_INTERFACE, "Interface %s address data initialized",
           ifdata->name);
     }
+    _trigger_if_change_including_any(ifdata);
   }
-
-  _trigger_if_change_including_any(ifdata);
 }
 
 /**
@@ -1465,7 +1536,7 @@ _cb_cfg_changed(void) {
       if (!data->_internal.ignore_mesh
          && data->_internal.mesh_counter > 0) {
         /* reactivate mesh settings */
-        _init_mesh(data);
+        _cleanup_mesh(data);
       }
 
       /* remove allocated instance (if no listener is left) */
